/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
.yui-h-slider,
.yui-v-slider {
    position: relative;
}
.yui-h-slider .yui-slider-thumb,
.yui-v-slider .yui-slider-thumb {
    position: absolute;
    cursor: default;
}
