/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
.yui-carousel {
    visibility: hidden;
    overflow: hidden;
    position: relative;
    text-align: left;
    zoom: 1;
}

.yui-carousel.yui-carousel-visible {
    visibility: visible;
}

.yui-carousel-content {
    overflow: hidden;
    position: relative;
}

.yui-carousel-element {
    margin: 5px 0;
    overflow: hidden;
    padding: 0;
    position: relative;
    width: 32000px;
    z-index: 1;
}

.yui-carousel-vertical .yui-carousel-element {
    margin: 0 5px;
}

.yui-carousel-element li {
    border: 1px solid #ccc;
    float: left;
    list-style: none;
    margin: 1px;
    overflow: hidden;
    padding: 0;
    text-align: center;
    /* IE 6 & 7 fix - prevent DOM scroll for focussed elements. */
   *float: none;
   *display: inline-block;
   *zoom: 1;
   *display: inline;
}

.yui-carousel .yui-carousel-item-selected {
    border: 1px dashed #000;
    margin: 1px;
}

.yui-carousel-vertical {
    height: 32000px;
    margin: 0 5px;
    width: auto;
}

.yui-carousel-vertical .yui-carousel-element li {
    display: block;
    float: none;
}

.yui-log .carousel {
    background: #f2e886;
}

.yui-carousel-nav {
    zoom: 1;
}

.yui-carousel-nav:after {
    clear: both;
    content: "";
    display: block;
}

.yui-carousel-button-focus {
    outline: 1px dotted #000;
}

.yui-carousel-min-width .yui-carousel-content {
    margin: 0 auto;
}
