/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
.yui-skin-sam .yui-pg-container {
    display: block;
    margin: 6px 0;
    white-space: nowrap;
}
.yui-skin-sam .yui-pg-first,
.yui-skin-sam .yui-pg-previous,
.yui-skin-sam .yui-pg-next,
.yui-skin-sam .yui-pg-last,
.yui-skin-sam .yui-pg-current,
.yui-skin-sam .yui-pg-pages,
.yui-skin-sam .yui-pg-page {
    display: inline-block;
    font-family: arial,helvetica,clean,sans-serif;
    padding: 3px 6px;
    zoom:1;
}
.yui-skin-sam .yui-pg-pages {
    padding: 0;
}
.yui-skin-sam .yui-pg-current {
    padding: 3px 0;
}
.yui-skin-sam a.yui-pg-first:link,      .yui-skin-sam a.yui-pg-first:visited,
.yui-skin-sam a.yui-pg-first:active,    .yui-skin-sam a.yui-pg-first:hover,
.yui-skin-sam a.yui-pg-previous:link,   .yui-skin-sam a.yui-pg-previous:visited,
.yui-skin-sam a.yui-pg-previous:active, .yui-skin-sam a.yui-pg-previous:hover,
.yui-skin-sam a.yui-pg-next:link,       .yui-skin-sam a.yui-pg-next:visited,
.yui-skin-sam a.yui-pg-next:active,     .yui-skin-sam a.yui-pg-next:hover,
.yui-skin-sam a.yui-pg-last:link,       .yui-skin-sam a.yui-pg-last:visited,
.yui-skin-sam a.yui-pg-last:active,     .yui-skin-sam a.yui-pg-last:hover,
.yui-skin-sam a.yui-pg-page:link,       .yui-skin-sam a.yui-pg-page:visited,
.yui-skin-sam a.yui-pg-page:active,     .yui-skin-sam a.yui-pg-page:hover {
    color: #06c;
    text-decoration: underline;
    outline: 0;
}
.yui-skin-sam span.yui-pg-first,
.yui-skin-sam span.yui-pg-previous,
.yui-skin-sam span.yui-pg-next,
.yui-skin-sam span.yui-pg-last {
    color: #a6a6a6;
}
.yui-skin-sam .yui-pg-page {
    background-color: #fff;
    border: 1px solid #CBCBCB;
    padding: 2px 6px;
    text-decoration: none;
}
.yui-skin-sam .yui-pg-current-page {
    background-color: transparent;
    border: none;
    font-weight: bold;
    padding: 3px 6px;
}
.yui-skin-sam .yui-pg-page {
    margin-left: 1px;
    margin-right: 1px;
}
.yui-skin-sam .yui-pg-first,
.yui-skin-sam .yui-pg-previous {
    padding-left: 0;
}
.yui-skin-sam .yui-pg-next,
.yui-skin-sam .yui-pg-last {
    padding-right: 0;
}
.yui-skin-sam .yui-pg-current,
.yui-skin-sam .yui-pg-rpp-options {
    margin-left: 1em;
    margin-right: 1em;
}
