/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
.yui-skin-sam .yui-h-slider {
    background: url(bg-h.gif) no-repeat 5px 0;
    height: 28px;
    width: 228px;
}
.yui-skin-sam .yui-h-slider .yui-slider-thumb {
    top: 4px;
}

.yui-skin-sam .yui-v-slider {
    background: url(bg-v.gif) no-repeat 12px 0;
    height: 228px;
    width: 48px;
}
