/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
.yui-crop {
    position: relative;
}
.yui-crop .yui-crop-mask {
    position: absolute;
    top: 0;
    left: 0;
    height: 100%;
    width: 100%;
}

.yui-crop .yui-resize {
    position: absolute;
    top: 10px;
    left: 10px;
    border: 0;
}

.yui-crop .yui-crop-resize-mask {
    position: absolute;
    top: 0;
    left: 0;
    height: 100%;
    width: 100%;
    background-position: -10px -10px;
    overflow: hidden;
}
