/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
/*
Copyright (c) 2008, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.5.2
*/

/* the style of the div around each node */
.ygtvitem { }  

.ygtvitem table {
    margin-bottom:0; border:none;
}

/*.ygtvitem td {*/
.ygtvrow td {
    border: none; padding: 0;
}
.ygtvrow td a {
	text-decoration:none;
}


/* first or middle sibling, no children */
.ygtvtn {
    width:18px; height:22px; 
    background: url(treeview-sprite.gif) 0 -5600px no-repeat; 
}

/* first or middle sibling, collapsable */
.ygtvtm {
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -4000px no-repeat; 
}

/* first or middle sibling, collapsable, hover */
.ygtvtmh,.ygtvtmhh {
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -4800px no-repeat; 
}

/* first or middle sibling, expandable */
.ygtvtp {
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -6400px no-repeat; 
}

/* first or middle sibling, expandable, hover */
.ygtvtph ,.ygtvtphh {
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -7200px no-repeat; 
}

/* last sibling, no children */
.ygtvln {
    width:18px; height:22px; 
    background: url(treeview-sprite.gif) 0 -1600px no-repeat; 
}

/* Last sibling, collapsable */
.ygtvlm {
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 0px no-repeat; 
}

/* Last sibling, collapsable, hover */
.ygtvlmh,.ygtvlmhh {
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -800px no-repeat; 
}

/* Last sibling, expandable */
.ygtvlp { 
    width:18px; height:22px; 
    cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -2400px no-repeat; 
}

/* Last sibling, expandable, hover */
.ygtvlph,.ygtvlphh { 
    width:18px; height:22px; cursor:pointer ;
    background: url(treeview-sprite.gif) 0 -3200px no-repeat; 
}

/* Loading icon */
.ygtvloading { 
    width:18px; height:22px; 
    background: url(treeview-loading.gif) 0 0 no-repeat; 
}

/* the style for the empty cells that are used for rendering the depth 
 * of the node */
.ygtvdepthcell { 
    width:18px; height:22px; 
    background: url(treeview-sprite.gif) 0 -8000px no-repeat; 
}

.ygtvblankdepthcell { width:18px; height:22px; }


/* the style of the div around each node's collection of children */
.ygtvchildren {  }  
* html .ygtvchildren { height:2%; }  

/* the style of the text label in ygTextNode */
.ygtvlabel, .ygtvlabel:link, .ygtvlabel:visited, .ygtvlabel:hover { 
    margin-left:2px;
    text-decoration: none;
    background-color: white; /* workaround for IE font smoothing bug */
	cursor:pointer;
}

.ygtvcontent {
	cursor:default;
}

.ygtvspacer { height: 22px; width: 12px; }

.ygtvfocus {
	background-color: #c0e0e0;
	border: none;
}
.ygtvfocus .ygtvlabel, .ygtvfocus .ygtvlabel:link, .ygtvfocus .ygtvlabel:visited, .ygtvfocus .ygtvlabel:hover {
	background-color: #c0e0e0;
}

.ygtvfocus a , .ygtvrow  td a {
	outline-style:none;
}


.ygtvok {
    width:18px; height:22px; 
    background: url(treeview-sprite.gif) 0 -8800px no-repeat; 
}

.ygtvok:hover {
    background: url(treeview-sprite.gif) 0 -8844px no-repeat; 
}
	
.ygtvcancel {
    width:18px; height:22px; 
    background: url(treeview-sprite.gif) 0 -8822px no-repeat; 
}

.ygtvcancel:hover  {
    background: url(treeview-sprite.gif) 0 -8866px no-repeat; 
}

.ygtv-label-editor {
	background-color:#f2f2f2;
	border: 1px solid silver;
	position:absolute;
	display:none;
	overflow:hidden;
	margin:auto;
	z-index:9000;
}

.ygtv-edit-TextNode  {
	width: 190px;
}

.ygtv-edit-TextNode .ygtvcancel, .ygtv-edit-TextNode .ygtvok  {
	border:none;
}

.ygtv-edit-TextNode .ygtv-button-container {
	float: right;
}

.ygtv-edit-TextNode .ygtv-input  input{
	width: 140px;
}

.ygtv-edit-DateNode .ygtvcancel {
	border:none;
}
.ygtv-edit-DateNode .ygtvok  {
	display:none;
}

.ygtv-edit-DateNode   .ygtv-button-container {
	text-align:right;
	margin:auto;
}
