from hurry.resource import Library, ResourceInclusion

yui = Library('yui', 'yui-build')

fonts = ResourceInclusion(yui, 'fonts/fonts.css', minified='fonts/fonts-min.css')
grids = ResourceInclusion(yui, 'grids/grids.css', depends=[fonts], minified='grids/grids-min.css')
sam = ResourceInclusion(yui, 'assets/skins/sam/skin.css')
reset = ResourceInclusion(yui, 'reset/reset.css', minified='reset/reset-min.css')
reset_fonts = ResourceInclusion(yui, 'reset-fonts/reset-fonts.css', supersedes=[reset, fonts], minified='reset-fonts/reset-fonts.css')
reset_fonts_grids = ResourceInclusion(yui, 'reset-fonts-grids/reset-fonts-grids.css', supersedes=[reset, fonts, grids], minified='reset-fonts-grids/reset-fonts-grids.css')
base = ResourceInclusion(yui, 'base/base.css', minified='base/base-min.css')
yahoo = ResourceInclusion(yui, 'yahoo/yahoo.js', debug='yahoo/yahoo-debug.js', minified='yahoo/yahoo-min.js')
event = ResourceInclusion(yui, 'event/event.js', depends=[yahoo], debug='event/event-debug.js', minified='event/event-min.js')
dom = ResourceInclusion(yui, 'dom/dom.js', depends=[yahoo], debug='dom/dom-debug.js', minified='dom/dom-min.js')
imageloader = ResourceInclusion(yui, 'imageloader/imageloader.js', depends=[event, dom], debug='imageloader/imageloader-debug.js', minified='imageloader/imageloader-min.js')
containercore = ResourceInclusion(yui, 'container/container_core.js', depends=[dom, event], debug='container/container_core-debug.js', minified='container/container_core-min.js')
calendar = ResourceInclusion(yui, 'calendar/calendar.js', depends=[event, dom], debug='calendar/calendar-debug.js', minified='calendar/calendar-min.js')
profiler = ResourceInclusion(yui, 'profiler/profiler.js', depends=[yahoo], debug='profiler/profiler-debug.js', minified='profiler/profiler-min.js')
element = ResourceInclusion(yui, 'element/element.js', depends=[dom, event], debug='element/element-debug.js', minified='element/element-min.js')
paginator = ResourceInclusion(yui, 'paginator/paginator.js', depends=[element], debug='paginator/paginator-debug.js', minified='paginator/paginator-min.js')
container = ResourceInclusion(yui, 'container/container.js', depends=[dom, event], supersedes=[containercore], debug='container/container.js', minified='container/container.js')
simpleeditor = ResourceInclusion(yui, 'editor/simpleeditor.js', depends=[element], debug='editor/simpleeditor-debug.js', minified='editor/simpleeditor-min.js')
menu = ResourceInclusion(yui, 'menu/menu.js', depends=[containercore], debug='menu/menu-debug.js', minified='menu/menu-min.js')
stylesheet = ResourceInclusion(yui, 'stylesheet/stylesheet.js', depends=[yahoo], debug='stylesheet/stylesheet-debug.js', minified='stylesheet/stylesheet-min.js')
json = ResourceInclusion(yui, 'json/json.js', depends=[yahoo], debug='json/json-debug.js', minified='json/json-min.js')
datasource = ResourceInclusion(yui, 'datasource/datasource.js', depends=[event], debug='datasource/datasource-debug.js', minified='datasource/datasource-min.js')
charts = ResourceInclusion(yui, 'charts/charts.js', depends=[element, json, datasource], debug='charts/charts-debug.js', minified='charts/charts-min.js')
button = ResourceInclusion(yui, 'button/button.js', depends=[element], debug='button/button-debug.js', minified='button/button-min.js')
editor = ResourceInclusion(yui, 'editor/editor.js', depends=[menu, element, button], supersedes=[simpleeditor], debug='editor/editor.js', minified='editor/editor.js')
get = ResourceInclusion(yui, 'get/get.js', depends=[yahoo], debug='get/get-debug.js', minified='get/get-min.js')
yuiloader_dom_event = ResourceInclusion(yui, 'yuiloader-dom-event/yuiloader-dom-event.js', supersedes=[yahoo, dom, event, get], debug='yuiloader-dom-event/yuiloader-dom-event.js', minified='yuiloader-dom-event/yuiloader-dom-event.js')
animation = ResourceInclusion(yui, 'animation/animation.js', depends=[dom, event], debug='animation/animation-debug.js', minified='animation/animation-min.js')
dragdrop = ResourceInclusion(yui, 'dragdrop/dragdrop.js', depends=[dom, event], debug='dragdrop/dragdrop-debug.js', minified='dragdrop/dragdrop-min.js')
carousel = ResourceInclusion(yui, 'carousel/carousel.js', depends=[element], debug='carousel/carousel-debug.js', minified='carousel/carousel-min.js')
logger = ResourceInclusion(yui, 'logger/logger.js', depends=[event, dom], debug='logger/logger-debug.js', minified='logger/logger-min.js')
slider = ResourceInclusion(yui, 'slider/slider.js', depends=[dragdrop], debug='slider/slider-debug.js', minified='slider/slider-min.js')
colorpicker = ResourceInclusion(yui, 'colorpicker/colorpicker.js', depends=[slider, element], debug='colorpicker/colorpicker-debug.js', minified='colorpicker/colorpicker-min.js')
resize = ResourceInclusion(yui, 'resize/resize.js', depends=[dom, event, dragdrop, element], debug='resize/resize-debug.js', minified='resize/resize-min.js')
imagecropper = ResourceInclusion(yui, 'imagecropper/imagecropper.js', depends=[dom, event, dragdrop, element, resize], debug='imagecropper/imagecropper-debug.js', minified='imagecropper/imagecropper-min.js')
yuitest = ResourceInclusion(yui, 'yuitest/yuitest.js', depends=[logger], debug='yuitest/yuitest-debug.js', minified='yuitest/yuitest-min.js')
connection = ResourceInclusion(yui, 'connection/connection.js', depends=[event], debug='connection/connection-debug.js', minified='connection/connection-min.js')
utilities = ResourceInclusion(yui, 'utilities/utilities.js', supersedes=[yahoo, event, dragdrop, animation, dom, connection, element, get], debug='utilities/utilities.js', minified='utilities/utilities.js')
cookie = ResourceInclusion(yui, 'cookie/cookie.js', depends=[yahoo], debug='cookie/cookie-debug.js', minified='cookie/cookie-min.js')
uploader = ResourceInclusion(yui, 'uploader/uploader.js', depends=[element], debug='uploader/uploader-debug.js', minified='uploader/uploader-min.js')
layout = ResourceInclusion(yui, 'layout/layout.js', depends=[dom, event, element], debug='layout/layout-debug.js', minified='layout/layout-min.js')
yuiloader = ResourceInclusion(yui, 'yuiloader/yuiloader.js', supersedes=[yahoo, get], debug='yuiloader/yuiloader.js', minified='yuiloader/yuiloader.js')
profilerviewer = ResourceInclusion(yui, 'profilerviewer/profilerviewer.js', depends=[profiler, yuiloader, element], debug='profilerviewer/profilerviewer-debug.js', minified='profilerviewer/profilerviewer-min.js')
datatable = ResourceInclusion(yui, 'datatable/datatable.js', depends=[element, datasource], debug='datatable/datatable-debug.js', minified='datatable/datatable-min.js')
selector = ResourceInclusion(yui, 'selector/selector.js', depends=[yahoo, dom], debug='selector/selector-debug.js', minified='selector/selector-min.js')
tabview = ResourceInclusion(yui, 'tabview/tabview.js', depends=[element], debug='tabview/tabview-debug.js', minified='tabview/tabview-min.js')
autocomplete = ResourceInclusion(yui, 'autocomplete/autocomplete.js', depends=[dom, event, datasource], debug='autocomplete/autocomplete-debug.js', minified='autocomplete/autocomplete-min.js')
yahoo_dom_event = ResourceInclusion(yui, 'yahoo-dom-event/yahoo-dom-event.js', supersedes=[yahoo, event, dom], debug='yahoo-dom-event/yahoo-dom-event.js', minified='yahoo-dom-event/yahoo-dom-event.js')
treeview = ResourceInclusion(yui, 'treeview/treeview.js', depends=[event, dom], debug='treeview/treeview-debug.js', minified='treeview/treeview-min.js')
history = ResourceInclusion(yui, 'history/history.js', depends=[event], debug='history/history-debug.js', minified='history/history-min.js')