/*
Copyright (c) 2009, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.7.0
*/
.yui-skin-sam .yui-button  {

    border-width: 1px 0;
    border-style: solid;
    border-color: #808080;
    background: url(../../../../assets/skins/sam/sprite.png) repeat-x 0 0;
    margin: auto .25em;
    
}

.yui-skin-sam .yui-button .first-child {

    border-width: 0 1px;
    border-style: solid;
    border-color: #808080;
    margin: 0 -1px;

	/*
		Using negative margins for rounded corners won't work in IE 6 and IE 7 
		(Quirks Mode Only), so set the "margin" property to "0" for those 
		browsers.
	*/
 	_margin: 0;

}

.yui-skin-sam .yui-button button,
.yui-skin-sam .yui-button a {

    padding: 0 10px;
    font-size: 93%;  /* 12px */
    line-height: 2;  /* ~24px */
    *line-height: 1.7; /* For IE */
    min-height: 2em; /* For Gecko */
    *min-height: auto; /* For IE */
    color: #000; 

}

.yui-skin-sam .yui-button a {

    /*
        Necessary to get Buttons of type "link" to be the correct 
        height in IE.
    */
    *line-height: 1.875;
	*padding-bottom: 1px;

}

.yui-skin-sam .yui-split-button button,
.yui-skin-sam .yui-menu-button button {

    padding-right: 20px;
    background-position: right center;
    background-repeat: no-repeat;
    
}

.yui-skin-sam .yui-menu-button button {

    background-image: url(menu-button-arrow.png);

}

.yui-skin-sam .yui-split-button button {

    background-image: url(split-button-arrow.png);

}


/* Focus state */


.yui-skin-sam .yui-button-focus {

    border-color: #7D98B8;
    background-position: 0 -1300px;

}

.yui-skin-sam .yui-button-focus .first-child {

    border-color: #7D98B8;

}

.yui-skin-sam .yui-button-focus button, 
.yui-skin-sam .yui-button-focus a {

    color: #000;

}

.yui-skin-sam .yui-split-button-focus button {

    background-image: url(split-button-arrow-focus.png);

}


/* Hover state */

.yui-skin-sam .yui-button-hover {

    border-color: #7D98B8;
    background-position: 0 -1300px;

}

.yui-skin-sam .yui-button-hover .first-child {

    border-color: #7D98B8;

}

.yui-skin-sam .yui-button-hover button, 
.yui-skin-sam .yui-button-hover a {

    color: #000;

}

.yui-skin-sam .yui-split-button-hover button {

    background-image: url(split-button-arrow-hover.png);

}


/* Active state */

.yui-skin-sam .yui-button-active {
    
    border-color: #7D98B8;
    background-position: 0 -1700px;
    
}

.yui-skin-sam .yui-button-active .first-child {

    border-color: #7D98B8;

}

.yui-skin-sam .yui-button-active button, 
.yui-skin-sam .yui-button-active a {

    color: #000;

}

.yui-skin-sam .yui-split-button-activeoption {

    border-color: #808080;
    background-position: 0 0;

}

.yui-skin-sam .yui-split-button-activeoption .first-child {

    border-color: #808080;

}

.yui-skin-sam .yui-split-button-activeoption button {

    background-image: url(split-button-arrow-active.png);

}


/* Checked state */

.yui-skin-sam .yui-radio-button-checked,
.yui-skin-sam .yui-checkbox-button-checked {
    
    border-color: #304369;
    background-position: 0 -1400px;
    
}

.yui-skin-sam .yui-radio-button-checked .first-child,
.yui-skin-sam .yui-checkbox-button-checked .first-child {

    border-color: #304369;

}

.yui-skin-sam .yui-radio-button-checked button,
.yui-skin-sam .yui-checkbox-button-checked button { 

    color: #fff;

}


/* Disabled state */

.yui-skin-sam .yui-button-disabled {
    
    border-color: #ccc;
    background-position: 0 -1500px;
    
}

.yui-skin-sam .yui-button-disabled .first-child {

    border-color: #ccc;

}

.yui-skin-sam .yui-button-disabled button, 
.yui-skin-sam .yui-button-disabled a {

    color: #A6A6A6;
    cursor: default;

}

.yui-skin-sam .yui-menu-button-disabled button {

    background-image: url(menu-button-arrow-disabled.png);
    
}

.yui-skin-sam .yui-split-button-disabled button {

    background-image: url(split-button-arrow-disabled.png);
    
}